import { Component, Injectable, Input, NgModule, ViewChild } from '@angular/core';
import { axisBottom, axisLeft, easeLinear, event, extent, max, scaleBand, scaleLinear, scaleTime, select, timeFormat } from 'd3';
import { CommonModule } from '@angular/common';

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class ChartService {
    /**
     * Method to provide a chart canvas for d3 to operate on
     *
     *
     * @param {?} element
     * @param {?} width
     * @param {?} height
     * @param {?} margins
     * @return {?}
     */
    makeChartCanvas(element, width, height, margins) {
        const /** @type {?} */ el = element.nativeElement;
        return select(el)
            .append('svg')
            .attr('width', width)
            .attr('height', height)
            .append('g')
            .attr('class', 'chart-canvas')
            .attr('transform', `translate(${margins['left']}, ${margins['top']})`);
    }
}
ChartService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
ChartService.ctorParameters = () => [];

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class DataService {
    constructor() {
    }
    /**
     * This is a method to parse out the data to be visualized
     *
     *
     * @param {?} data
     * @param {?} dataElement
     * @return {?}
     */
    getData(data, dataElement) {
        return data.map((d) => d[dataElement]);
    }
    /**
     * This method creates any kind of d3 scale using builder pattern
     *
     *
     * @param {?} scaleType
     * @param {?} data
     * @param {?} measure
     * @param {?=} ordered
     * @return {?}
     */
    makeScale(scaleType, data, measure, ordered = true) {
        const /** @type {?} */ scaleOptions = {
            'linear': this.makeLinearScale(data, measure, ordered),
            'categorical': this.makeCategoricalScale(data, measure, ordered),
            'time': this.makeTimeScale(data, measure, ordered)
        };
        return scaleOptions[scaleType];
    }
    /**
     * @param {?} data
     * @param {?} measure
     * @param {?=} ordered
     * @return {?}
     */
    makeLinearScale(data, measure, ordered = true) {
        if (ordered) {
            return scaleLinear().domain([0, max(data)]).rangeRound([0, measure]);
        }
        return scaleLinear().domain([0, max(data)]).rangeRound([measure, 0]);
    }
    /**
     * @param {?} data
     * @param {?} measure
     * @param {?=} ordered
     * @return {?}
     */
    makeCategoricalScale(data, measure, ordered = true) {
        if (ordered) {
            return scaleBand().domain(data).rangeRound([0, measure]);
        }
        else {
            return scaleBand().domain(data).rangeRound([0, measure]);
        }
    }
    /**
     * @param {?} data
     * @param {?} measure
     * @param {?=} ordered
     * @return {?}
     */
    makeTimeScale(data, measure, ordered = true) {
        if (ordered) {
            return scaleTime().domain(extent(data)).rangeRound([0, measure]);
        }
        else {
            return scaleTime().domain(data).rangeRound([0, measure]);
        }
    }
}
DataService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
DataService.ctorParameters = () => [];

class Tooltip {
    /**
     * @param {?} xData
     * @param {?} yData
     * @param {?=} message
     */
    constructor(xData, yData, message = null) {
        this.x = xData;
        this.y = yData;
        this.message = message;
    }
}

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class ToolTipService {
    /**
     * A method to add tooltips to a d3 selection object, tooltip will show x, y datapoints and
     * any meta data mentioned in message property
     *
     *
     * @param {?} selection
     * @param {?} labelKeys
     * @param {?} refElement
     * @return {?}
     */
    addTooltip(selection, labelKeys, refElement) {
        const /** @type {?} */ tooltipBoxContainer = refElement.nativeElement;
        const /** @type {?} */ tooltipBox = select(tooltipBoxContainer)
            .append('div')
            .attr('class', 'tooltip')
            .style('opacity', 0);
        return selection
            .on('mouseover', (d) => {
            const /** @type {?} */ content = `${labelKeys.x}: ${d[labelKeys.x]}<br/>${labelKeys.y}: ${d[labelKeys.y]}`;
            let /** @type {?} */ box;
            tooltipBox
                .transition()
                .duration(200)
                .style('opacity', 0.95);
            if (labelKeys.message !== null) {
                const /** @type {?} */ message = `${labelKeys.message}: ${d[labelKeys.message]}`;
                box = tooltipBox.html(`<p>${content}<br/>${message}</p>`);
            }
            else {
                box = tooltipBox.html(`<p>${content}</p>`);
            }
            box
                .style('left', `${event.pageX}px`)
                .style('top', `${event.pageY}px`)
                .style('position', 'absolute')
                .style('text-align', 'center')
                .style('width', '6%')
                .style('height', '8%')
                .style('padding', '5px')
                .style('line-height', 1)
                .style('font-family', 'sans-serif')
                .style('font-size', '1em')
                .style('color', '#fff')
                .style('background', 'lightsteelblue')
                .style('border', 0)
                .style('word-wrap', 'break-word')
                .style('overflow', 'hidden')
                .style('border-radius', '8px')
                .style('pointer-events', 'none');
        })
            .on('mouseout', (d) => {
            tooltipBox.transition()
                .duration(500)
                .style('opacity', 0);
        });
    }
}
ToolTipService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
ToolTipService.ctorParameters = () => [];

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class BarChartComponent {
    /**
     * @param {?} chartService
     * @param {?} dataService
     * @param {?} tooltipService
     */
    constructor(chartService, dataService, tooltipService) {
        this.chartService = chartService;
        this.dataService = dataService;
        this.tooltipService = tooltipService;
        /**
         * Obtain the settings for making the chart here and define defaults if none are specified.
         */
        this.width = 400;
        this.height = 400;
        this.margins = { 'top': 60, 'right': 60, 'bottom': 60, 'left': 60 };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /*Check if data is given if not die*/
        if (this.data == null) {
            throw new Error('Missing Data!');
        }
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        this.chart = this.chartService.makeChartCanvas(this.chartContainer, this.width, this.height, this.margins);
        if (this.data) {
            this.draw(this.data);
        }
    }
    /**
     * This life cycle hook tells angular to detect any changes and to adjust
     * the chart accordingly and also serves to separate out different
     * instances of the same chart
     * @return {?}
     */
    ngOnChanges() {
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        if (this.chart && this.data) {
            this.draw(this.data);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.width = null;
        this.height = null;
        this.margins = null;
        this.data = null;
        this.x = null;
        this.y = null;
        this.chartHeight = null;
        this.chartWidth = null;
        this.chart = null;
        this.xScale = null;
        this.yScale = null;
        this.xData = null;
        this.yData = null;
    }
    /**
     * This makes the axes for the chart.
     *
     *
     * @param {?} axis
     * @param {?} scale
     * @return {?}
     */
    makeAxis(axis, scale) {
        if (axis === 'x') {
            /* This is drawing the x-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'x-axis')
                .attr('transform', `translate(0, ${this.chartHeight})`)
                .call(axisBottom(scale));
            const /** @type {?} */ movement = `${this.chartWidth / 2}, ${this.chartHeight + this.margins.bottom - 5}`;
            this.chart
                .append('text')
                .attr('transform', `translate(${movement})`)
                .text(`${this.x}`);
        }
        else if (axis === 'y') {
            /* This is drawing the y-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'y-axis')
                .call(axisLeft(scale));
            this.chart.append('text')
                .attr('transform', 'rotate(-90)')
                .attr('y', `-${this.margins.left}`)
                .attr('x', `-${this.chartHeight / 2}`)
                .attr('dy', '1em')
                .style('text-anchor', 'middle')
                .text(`${this.y}`);
        }
    }
    /**
     * This is the main method to render the bar chart it operates on the d3
     * selection supplied and appends the scales, bars to the selection.
     *
     * @param {?} dataSet
     * @return {?}
     */
    draw(dataSet) {
        this.xData = this.dataService.getData(dataSet, this.x);
        this.yData = this.dataService.getData(dataSet, this.y);
        this.xScale = this.dataService.makeScale('categorical', this.xData, this.chartWidth);
        this.yScale = this.dataService.makeScale('linear', this.yData, this.chartHeight, false);
        this.makeAxis('x', this.xScale);
        this.makeAxis('y', this.yScale);
        /**
         * We draw the bars into the canvas here. For transitions, the initial height and y have to
         * be set to the bottom of the chart before rendering the actual data. We animate a rising
         * effect, the placement of the transition logic drives what actually animates.
         */
        const svg = this.chart.selectAll('.bar')
            .data(dataSet)
            .enter()
            .append('rect')
            .attr('class', 'bar')
            .attr('width', this.xScale.bandwidth())
            .attr('x', d => this.xScale(d[this.x]))
            .attr('y', this.yScale(0))
            .attr('height', this.chartHeight - this.yScale(0));
        this.tooltipService.addTooltip(svg, this.tooltipLabels, this.chartContainer);
        svg
            .transition()
            .ease(easeLinear)
            .duration(880)
            .attr('y', d => this.yScale(d[this.y]))
            .attr('height', d => this.chartHeight - this.yScale(d[this.y]));
    }
}
BarChartComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngz-charts-barchart',
                template: `
    <div #barChart class="canvas"></div>
  `,
                styles: [`
    .canvas {
      height: 100%;
      width: 100%; }

    ::ng-deep .bar {
      fill: steelblue; }

    ::ng-deep .x-axis, .y-axis {
      font-family: 'Open Sans', sans-serif;
      font-size: 100%;
      font-weight: 400; }
  `]
            },] },
];
/**
 * @nocollapse
 */
BarChartComponent.ctorParameters = () => [
    { type: ChartService, },
    { type: DataService, },
    { type: ToolTipService, },
];
BarChartComponent.propDecorators = {
    'width': [{ type: Input },],
    'height': [{ type: Input },],
    'margins': [{ type: Input },],
    'data': [{ type: Input },],
    'x': [{ type: Input },],
    'y': [{ type: Input },],
    'chartContainer': [{ type: ViewChild, args: ['barChart',] },],
};

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class SharedModule {
}
SharedModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                providers: [ChartService, DataService, ToolTipService]
            },] },
];
/**
 * @nocollapse
 */
SharedModule.ctorParameters = () => [];

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class BarChartModule {
}
BarChartModule.decorators = [
    { type: NgModule, args: [{
                imports: [SharedModule],
                declarations: [
                    BarChartComponent,
                ],
                exports: [
                    BarChartComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
BarChartModule.ctorParameters = () => [];

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class ScatterPlotComponent {
    /**
     * @param {?} chartService
     * @param {?} dataService
     * @param {?} tooltipService
     */
    constructor(chartService, dataService, tooltipService) {
        this.chartService = chartService;
        this.dataService = dataService;
        this.tooltipService = tooltipService;
        /**
         * Setup the chart properties and define defaults if none are specified
         */
        this.width = 600;
        this.height = 600;
        this.margins = { 'top': 50, 'right': 50, 'bottom': 50, 'left': 50 };
        this.dotSize = 5;
        this.dataPointShape = 'circle';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.data === null) {
            throw new Error('No Data supplied!!');
        }
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        this.chart = this.chartService.makeChartCanvas(this.chartContainer, this.width, this.height, this.margins);
        if (this.data) {
            this.draw(this.data);
        }
    }
    /**
     * This life cycle hook tells angular to detect any changes and to adjust
     * the chart accordingly and also serves to separate out different
     * instances of the same chart
     *
     * @return {?}
     */
    ngOnChanges() {
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        if (this.chart && this.data) {
            this.draw(this.data);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.width = null;
        this.height = null;
        this.margins = null;
        this.data = null;
        this.x = null;
        this.y = null;
        this.chartHeight = null;
        this.chartWidth = null;
        this.chart = null;
        this.xScale = null;
        this.yScale = null;
        this.xData = null;
        this.yData = null;
    }
    /**
     * Main function to draw the scatter plot, it requires a dataset in order to draw
     * on the reference element supplied from view child
     *
     * \@param{Array<object>} dataSet - data objects in an array.
     * @param {?} dataSet
     * @return {?}
     */
    draw(dataSet) {
        this.xData = this.dataService.getData(dataSet, this.x);
        this.yData = this.dataService.getData(dataSet, this.y);
        this.xScale = this.dataService.makeScale('linear', this.xData, this.chartWidth);
        this.yScale = this.dataService.makeScale('linear', this.yData, this.chartHeight);
        this.makeAxis('x', this.xScale);
        this.makeAxis('y', this.yScale);
        const /** @type {?} */ svg = this.chart.selectAll('.scatter-plot-data')
            .data(dataSet)
            .enter()
            .append(this.dataPointShape)
            .attr('class', 'scatter-plot-data')
            .attr('r', this.dotSize)
            .attr('cx', (d) => this.xScale(d[this.x]))
            .attr('cy', (d) => this.yScale(d[this.y]));
        this.tooltipService.addTooltip(svg, this.tooltipLabels, this.chartContainer);
    }
    /**
     * This makes the axes for the chart.
     *
     *
     * @param {?} axis
     * @param {?} scale
     * @return {?}
     */
    makeAxis(axis, scale) {
        if (axis === 'x') {
            /* This is drawing the x-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'x-axis')
                .attr('transform', `translate(0, ${this.chartHeight})`)
                .call(axisBottom(scale));
            const /** @type {?} */ movement = `${this.chartWidth / 2}, ${this.chartHeight + this.margins.bottom - 5}`;
            this.chart
                .append('text')
                .attr('transform', `translate(${movement})`)
                .text(`${this.x}`);
        }
        else if (axis === 'y') {
            /* This is drawing the y-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'y-axis')
                .call(axisLeft(scale));
            this.chart.append('text')
                .attr('transform', 'rotate(-90)')
                .attr('y', `-${this.margins.left}`)
                .attr('x', `-${this.chartHeight / 2}`)
                .attr('dy', '1em')
                .style('text-anchor', 'middle')
                .text(`${this.y}`);
        }
    }
}
ScatterPlotComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngz-charts-scatterplot',
                template: `
    <div #scatterPlot class="scatterplot-canvas"></div>
  `,
                styles: [`
    ::ng-deep .scatterplot-data {
      fill: steelblue; }
  `]
            },] },
];
/**
 * @nocollapse
 */
ScatterPlotComponent.ctorParameters = () => [
    { type: ChartService, },
    { type: DataService, },
    { type: ToolTipService, },
];
ScatterPlotComponent.propDecorators = {
    'width': [{ type: Input },],
    'height': [{ type: Input },],
    'margins': [{ type: Input },],
    'data': [{ type: Input },],
    'x': [{ type: Input },],
    'y': [{ type: Input },],
    'dotSize': [{ type: Input },],
    'dataPointShape': [{ type: Input },],
    'chartContainer': [{ type: ViewChild, args: ['scatterPlot',] },],
};

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class ScatterPlotModule {
}
ScatterPlotModule.decorators = [
    { type: NgModule, args: [{
                imports: [SharedModule],
                declarations: [ScatterPlotComponent],
                exports: [ScatterPlotComponent]
            },] },
];
/**
 * @nocollapse
 */
ScatterPlotModule.ctorParameters = () => [];

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class HorizontalBarChartComponent {
    /**
     * @param {?} chartService
     * @param {?} dataService
     * @param {?} tooltipService
     */
    constructor(chartService, dataService, tooltipService) {
        this.chartService = chartService;
        this.dataService = dataService;
        this.tooltipService = tooltipService;
        /**
         * Obtain the settings for making the chart here and define defaults if none are specified.
         */
        this.width = 400;
        this.height = 400;
        this.margins = { 'top': 80, 'right': 80, 'bottom': 80, 'left': 80 };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /*Check if data is given if not die*/
        if (this.data == null) {
            throw new Error('Missing Data!');
        }
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        this.chart = this.chartService.makeChartCanvas(this.chartContainer, this.width, this.height, this.margins);
        if (this.data) {
            this.draw(this.data);
        }
    }
    /**
     * This life cycle hook tells angular to detect any changes and to adjust
     * the chart accordingly and also serves to separate out different
     * instances of the same chart
     *
     * @return {?}
     */
    ngOnChanges() {
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        if (this.chart && this.data) {
            this.draw(this.data);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.width = null;
        this.height = null;
        this.margins = null;
        this.data = null;
        this.x = null;
        this.y = null;
        this.chartHeight = null;
        this.chartWidth = null;
        this.chart = null;
        this.xScale = null;
        this.yScale = null;
        this.xData = null;
        this.yData = null;
    }
    /**
     * This makes the axes for the chart.
     *
     *
     * @param {?} axis
     * @param {?} scale
     * @return {?}
     */
    makeAxis(axis, scale) {
        if (axis === 'x') {
            /* This is drawing the x-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'x-axis')
                .attr('transform', `translate(0, ${this.chartHeight})`)
                .call(axisBottom(scale));
            const /** @type {?} */ movement = `${this.chartWidth / 2}, ${this.chartHeight + this.margins.bottom - 5}`;
            this.chart
                .append('text')
                .attr('transform', `translate(${movement})`)
                .text(`${this.x}`);
        }
        else if (axis === 'y') {
            /* This is drawing the y-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'y-axis')
                .call(axisLeft(scale));
            this.chart.append('text')
                .attr('transform', 'rotate(-90)')
                .attr('y', `-${this.margins.left}`)
                .attr('x', `-${this.chartHeight / 2}`)
                .attr('dy', '1em')
                .style('text-anchor', 'middle')
                .text(`${this.y}`);
        }
    }
    /**
     * This is the main method to render the bar chart it operates on the d3
     * selection supplied and appends the scales, bars to the selection.
     *
     * @param {?} dataSet
     * @return {?}
     */
    draw(dataSet) {
        this.xData = this.dataService.getData(dataSet, this.x);
        this.yData = this.dataService.getData(dataSet, this.y);
        this.yScale = this.dataService.makeScale('categorical', this.yData, this.chartHeight, false);
        this.xScale = this.dataService.makeScale('linear', this.xData, this.chartWidth);
        this.makeAxis('x', this.xScale);
        this.makeAxis('y', this.yScale);
        /**
         * We draw the bars into the canvas here. For transitions, the initial height and y have to
         * be set to the bottom of the chart before rendering the actual data. We animate a rising
         * effect, the placement of the transition logic drives what actually animates.
         */
        const svg = this.chart.selectAll('.h-bar')
            .data(dataSet).enter().append('rect')
            .attr('class', 'h-bar')
            .attr('height', this.yScale.bandwidth())
            .attr('y', d => this.yScale(d[this.y]))
            .attr('x', this.xScale(0))
            .attr('width', this.xScale(0));
        svg.transition()
            .duration(800)
            .ease(easeLinear)
            .attr('width', d => this.xScale(d[this.x]));
        this.tooltipService.addTooltip(svg, this.tooltipLabels, this.chartContainer);
    }
}
HorizontalBarChartComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngz-charts-horizontal-bar-chart',
                template: `
    <div #horizontalBarChart class="canvas"></div>
  `,
                styles: [`
    .canvas {
      height: 100%;
      width: 100%; }

    ::ng-deep .h-bar {
      fill: steelblue; }

    ::ng-deep .x-axis, .y-axis {
      font-family: 'Open Sans', sans-serif;
      font-size: 100%;
      font-weight: 400; }
  `]
            },] },
];
/**
 * @nocollapse
 */
HorizontalBarChartComponent.ctorParameters = () => [
    { type: ChartService, },
    { type: DataService, },
    { type: ToolTipService, },
];
HorizontalBarChartComponent.propDecorators = {
    'width': [{ type: Input },],
    'height': [{ type: Input },],
    'margins': [{ type: Input },],
    'data': [{ type: Input },],
    'x': [{ type: Input },],
    'y': [{ type: Input },],
    'chartContainer': [{ type: ViewChild, args: ['horizontalBarChart',] },],
};

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class HorizontalBarChartModule {
}
HorizontalBarChartModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    SharedModule
                ],
                declarations: [HorizontalBarChartComponent],
                exports: [HorizontalBarChartComponent]
            },] },
];
/**
 * @nocollapse
 */
HorizontalBarChartModule.ctorParameters = () => [];

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
class TimelineComponent {
    /**
     * @param {?} chartService
     * @param {?} dataService
     * @param {?} tooltipService
     */
    constructor(chartService, dataService, tooltipService) {
        this.chartService = chartService;
        this.dataService = dataService;
        this.tooltipService = tooltipService;
        /**
         * Obtain the settings for making the chart here and define defaults if none are specified.
         */
        this.width = 800;
        this.height = 400;
        this.margins = { 'top': 60, 'right': 60, 'bottom': 60, 'left': 60 };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /*Check if data is given if not die*/
        if (this.data == null) {
            throw new Error('Missing Data!');
        }
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.dateLabel, this.dateEvent);
        this.chart = this.chartService.makeChartCanvas(this.chartContainer, this.width, this.height, this.margins);
        if (this.data) {
            this.draw(this.data);
        }
    }
    /**
     * This life cycle hook tells angular to detect any changes and to adjust
     * the chart accordingly and also serves to separate out different
     * instances of the same chart
     * @return {?}
     */
    ngOnChanges() {
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.dateLabel, this.dateEvent);
        if (this.chart && this.data) {
            this.draw(this.data);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.width = null;
        this.height = null;
        this.margins = null;
        this.data = null;
        this.dateLabel = null;
        this.dateEvent = null;
        this.chartHeight = null;
        this.chartWidth = null;
        this.chart = null;
        this.data = null;
        this.timeScale = null;
    }
    /**
     * This makes the timeline axes
     *
     * @param {?} scale
     * @return {?}
     */
    makeTimeAxis(scale) {
        this.chart
            .append('g')
            .attr('class', 'time-axis')
            .attr('transform', `translate(0, ${this.chartHeight / 2})`)
            .call(axisBottom(scale).tickFormat(timeFormat('%m/%d/%Y')));
        const /** @type {?} */ axisLabelPos = `${this.chartWidth / 2}, ${this.chartHeight - this.margins.bottom}`;
        this.chart
            .append('text')
            .attr('transform', `translate(${axisLabelPos})`)
            .text(`${this.dateLabel}`);
    }
    /**
     * This is the main method to render the bar chart it operates on the d3
     * selection supplied and appends the scales, bars to the selection.
     *
     * @param {?} dataSet
     * @return {?}
     */
    draw(dataSet) {
        this.timeData = this.dataService.getData(dataSet, this.dateLabel);
        this.timeScale = this.dataService.makeScale('time', this.timeData, this.chartWidth);
        this.makeTimeAxis(this.timeScale);
        /**
         * We draw the bars into the canvas here. For transitions, the initial height and y have to
         * be set to the bottom of the chart before rendering the actual data. We animate a rising
         * effect, the placement of the transition logic drives what actually animates.
         */
        const svg = this.chart.selectAll('.bar')
            .data(dataSet)
            .enter()
            .append('circle')
            .attr('class', 'time-point')
            .attr('r', 8)
            .attr('cx', d => this.timeScale(d[this.dateLabel]))
            .attr('cy', `${this.chartHeight / 2}`);
        this.tooltipService.addTooltip(svg, this.tooltipLabels, this.chartContainer);
    }
}
TimelineComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngz-charts-timeline',
                template: `
    <div #timeline class="canvas"></div>
  `,
                styles: [`
    .canvas {
      width: 100%;
      height: 100%; }

    ::ng-deep .time-point {
      fill: steelblue; }

    ::ng-deep .time-axis {
      font-family: 'Open Sans', sans-serif;
      font-size: 100%;
      font-weight: 400; }
  `]
            },] },
];
/**
 * @nocollapse
 */
TimelineComponent.ctorParameters = () => [
    { type: ChartService, },
    { type: DataService, },
    { type: ToolTipService, },
];
TimelineComponent.propDecorators = {
    'width': [{ type: Input },],
    'height': [{ type: Input },],
    'margins': [{ type: Input },],
    'data': [{ type: Input },],
    'dateLabel': [{ type: Input },],
    'dateEvent': [{ type: Input },],
    'chartContainer': [{ type: ViewChild, args: ['timeline',] },],
};

class TimelineModule {
}
TimelineModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    SharedModule
                ],
                exports: [TimelineComponent],
                declarations: [TimelineComponent]
            },] },
];
/**
 * @nocollapse
 */
TimelineModule.ctorParameters = () => [];

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */

/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */

/**
 * Generated bundle index. Do not edit.
 */

export { BarChartModule, ScatterPlotModule, SharedModule, HorizontalBarChartModule, TimelineModule, BarChartComponent as ɵd, HorizontalBarChartComponent as ɵf, ScatterPlotComponent as ɵe, ChartService as ɵa, DataService as ɵb, ToolTipService as ɵc, TimelineComponent as ɵg };
//# sourceMappingURL=ngz-charts.js.map
